package com.onlinebank.transaction.interaction;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;

import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.mapping.MappingException;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;
import org.xml.sax.InputSource;

import com.onlinebank.transaction.to.TransactionSearchResult;
import com.onlinebank.util.exception.XMLException;

/**
 * This class contains the methods to generate XML from the transaction objects
 * that need to be returned to the client as part of the REST service response.
 * This class uses the Castor framework to generate the XML (see
 * TransactionMapping.xml for mapping configuration details).
 * 
 * @author Leon Rubin
 */
public class TransactionXMLGenerator {
	private static final String MAPPING_FILE_NAME = "TransactionMapping.xml";

	private static Mapping mapping;
	private static boolean initialized;

	/**
	 * Initializes the generator by loading the Castor mapping file.
	 * 
	 * @throws XMLException
	 *             if an error occurs while loading a Castor mapping file
	 */
	public static void init() throws XMLException {
		try {
			mapping = new Mapping();

			InputStream inputStream = TransactionXMLGenerator.class
					.getClassLoader().getResourceAsStream(MAPPING_FILE_NAME);

			if (inputStream == null) {
				throw new XMLException("Resource " + MAPPING_FILE_NAME
						+ " not found in classpath");
			}

			mapping.loadMapping(new InputSource(inputStream));

			initialized = true;
		} catch (IOException e) {
			e.printStackTrace();
			throw new XMLException("Failed to create mapping from file "
					+ MAPPING_FILE_NAME, e);
		} catch (MappingException e) {
			e.printStackTrace();
			throw new XMLException("Failed to create mapping from file "
					+ MAPPING_FILE_NAME, e);
		}
	}

	/**
	 * Generate the XML representation of the transaction search result object.
	 * 
	 * @param result
	 *            transaction search result object
	 * @return String containing the XML representation of the specified
	 *         transaction search result
	 * @throws XMLException
	 */
	public static String generate(TransactionSearchResult result)
			throws XMLException {
		try {
			if (!initialized) {
				init();
			}

			StringWriter stringWriter = new StringWriter();
			Marshaller marshaller = new Marshaller(stringWriter);

			marshaller.setMapping(mapping);
			marshaller.marshal(result);

			return stringWriter.toString();
		} catch (MarshalException e) {
			e.printStackTrace();
			throw new XMLException(
					"Failed to marshal a TransactionSearchResult", e);
		} catch (MappingException e) {
			e.printStackTrace();
			throw new XMLException(
					"A mapping error detected while marshalling a TransactionSearchResult",
					e);
		} catch (ValidationException e) {
			e.printStackTrace();
			throw new XMLException(
					"A validation error detected while marshalling a TransactionSearchResult",
					e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new XMLException(
					"An IO error detected while marshalling a TransactionSearchResult",
					e);
		}
	}
}
