package com.democompany.soa.esb.adaptor.rest.customj;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts2.dispatcher.FilterDispatcher;
import org.springframework.web.context.ContextLoaderListener;

import com.democompany.soa.esb.adaptor.util.customj.HttpXQServlet;
import com.democompany.soa.esb.adaptor.util.exception.ErrorCollection;
import com.democompany.soa.esb.adaptor.util.exception.ErrorCollectionXMLGenerator;
import com.democompany.soa.esb.adaptor.util.http.FilterConfigImpl;
import com.democompany.soa.esb.adaptor.util.http.HttpServletRequestImpl;
import com.democompany.soa.esb.adaptor.util.http.ServletContextImpl;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.util.OgnlValueStack;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQServiceException;

/**
 * This is the Custom Java Service class for the REST Adaptor. It builds on the
 * functionality defined in the HttpXQServlet.
 * 
 * @author Leon Rubin
 */
public class RESTAdaptorService extends HttpXQServlet {
	private static final String RESULT_XML_ATTRIBUTE = "resultXml";
	private static final String REST_CONTEXT_PATH = "/rest/";

	private ContextLoaderListener contextLoaderListener;
	private ServletContext servletContext;
	private ServletContextEvent servletContextEvent;
	private FilterDispatcher filterDispatcher;

	/**
	 * Constructor for the RESTAdaptorService
	 */
	public RESTAdaptorService() {
		super("RESTAdaptorService");
	}

	/**
	 * Initializes the Struts2 framework. Creates all the necessary components
	 * to run it in the ESB runtime container (basically in standalone mode
	 * outside of a web server). These components include the
	 * ContextLoaderListener necessary for the Spring framework that is used
	 * internally by Struts2,
	 * 
	 * @param initialContext
	 *            XQ initial context
	 * @throws XQServiceException
	 *             if any failures occur duing initialization
	 */
	@Override
	public void init(XQInitContext initialContext) throws XQServiceException {
		super.init(initialContext);

		try {
			initServletContext();
			initSpring();
			initStruts();
		} catch (Exception e) {
			throw new XQServiceException("Initialization failure", e);
		}
	}

	/**
	 * Initializes the servlet context so that it can be later passed to
	 * framework components that typically run in the context of a Servlet
	 * engine.
	 */
	private void initServletContext() {
		servletContext = new ServletContextImpl();
		servletContextEvent = new ServletContextEvent(servletContext);
	}

	/**
	 * Initializes the Struts2 framework (creates an instance of the
	 * FilterDispatcher which is an entry point for all struts action requests.
	 * It is an HTTP filter that is typically registered in a web.xml file, but
	 * since this code is running outside of a J2EE web server, the filter needs
	 * to be created and initialised programatically.
	 * 
	 * @throws ServletException
	 */
	private void initStruts() throws ServletException {
		filterDispatcher = new FilterDispatcher();

		FilterConfigImpl filterConfig = new FilterConfigImpl("struts2");
		filterConfig.setServletContext(servletContext);
		filterConfig.setInitParameterMap(new HashMap<String, String>());

		filterDispatcher.init(filterConfig);
	}

	/**
	 * Initializes the String framework (creates an instance of the
	 * ContextLoaderListener which is one of the common ways to configure a
	 * Spring application. It is an HTTP listener that is typically registered
	 * in a web.xml file, but since this code is running outside of a J2EE web
	 * server, the listener needs to be created and initialised programatically.
	 * 
	 * @throws ServletException
	 */
	private void initSpring() {
		contextLoaderListener = new ContextLoaderListener();
		contextLoaderListener.contextInitialized(servletContextEvent);
	}

	/**
	 * Extracts action errors, validation errors, and informational messages
	 * from the executed action. Errors are stored in the action object that is
	 * attached to the HTTP request as an attribute called struts.valueStack.
	 * Once the action is extracted, all errors can be copied to the error
	 * collection.
	 * 
	 * @param request
	 *            HTTP request containing the executed action
	 * @return ErrorCollection object which encapsulates errors that occured
	 *         during action execution.
	 */
	private ErrorCollection getErrors(HttpServletRequest request) {
		ErrorCollection errorCollection = new ErrorCollection();

		OgnlValueStack stack = (OgnlValueStack) request
				.getAttribute("struts.valueStack");

		if (stack != null) {
			for (int i = 0; i < stack.size(); i++) {
				Object object = stack.pop();

				if (object instanceof ActionSupport) {
					ActionSupport action = (ActionSupport) object;

					for (Object error : action.getFieldErrors().entrySet()) {
						errorCollection.addFieldError(error.toString());
					}

					for (Object error : action.getActionErrors()) {
						errorCollection.addError(error.toString());
					}

					for (Object message : action.getActionMessages()) {
						errorCollection.addMessages(message.toString());
					}
				}
			}
		}

		return errorCollection;
	}

	/**
	 * Process the GET request (since the service should respond in the same way
	 * regardless of the HTTP method used to send the information, this method
	 * simply calls the doPost method that actually contains the request
	 * processing logic.
	 */
	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) {
		doPost(request, response);
	}

	/**
	 * Handle the HTTP request by delegating the processing to the Struts2
	 * filter. The Struts2 filter will convert HTTP parameters into a Java model
	 * object (similar to a form bean in the old version of Struts) or populate
	 * input properties right on the action. The framework will then invoke the
	 * execute method on the action. The action class is responsible for
	 * executing the business logic. It is expected that the result of the
	 * action will be placed in a request attribute called resultXml. This
	 * method will retrieve the result XML and write it back to the client by
	 * writing it to the HttpServletResponse object.
	 */
	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) {
		String requestUrl = request.getRequestURL().toString();

		m_xqLog.logInformation("Request URL 27: " + requestUrl);

		if (requestUrl != null) {
			int startOfContextPath = requestUrl.indexOf(REST_CONTEXT_PATH);

			if (startOfContextPath > -1) {
				int queryStringStart = requestUrl
						.indexOf(HttpXQServlet.QUERY_STRING_SEPARATOR);
				String actionName = requestUrl.substring(startOfContextPath
						+ REST_CONTEXT_PATH.length() - 1, queryStringStart);
				m_xqLog.logInformation("Action: " + actionName);

				((HttpServletRequestImpl) request)
						.setServletPath(actionName);

				PrintWriter out = null;

				try {
					out = response.getWriter();
				} catch (IOException e1) {
					// This exception will never occur because we are not
					// working with a real
					// HTTP response but our custom implementation that does not
					// generate
					// this exception under any circumstances
					e1.printStackTrace();
				}

				try {
					filterDispatcher.doFilter(request, response, null);
					response.setContentType("text/xml");

					ErrorCollection errorCollection = getErrors(request);

					if (errorCollection.hasErrors()) {
						out.println(ErrorCollectionXMLGenerator
								.generate(errorCollection));
					} else {
						out.println((String) request
								.getAttribute(RESULT_XML_ATTRIBUTE));
					}
				} catch (Exception e) {
					e.printStackTrace();
					// If we are here then a system error occured and the only
					// type of error we
					// can send back is a hardcoded error message
					out.println("<error>Internal System Error " + e
							+ "</error>");
				}

				out.close();
			}
		}
	}

	/**
	 * Release all resources (specifically, notify the String framework that all
	 * resources allocated by it must be released). That is accomplished by
	 * sending the contextDestroyed event to the contextLoader listener
	 */
	public void destroy() {
		contextLoaderListener.contextDestroyed(servletContextEvent);
	}
}