package com.democompany.soa.esb.adaptor.util.customj;

import java.net.URLDecoder;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.democompany.soa.esb.adaptor.util.http.HttpServletRequestImpl;
import com.democompany.soa.esb.adaptor.util.http.HttpServletResponseImpl;
import com.sonicsw.xq.XQContext;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQPart;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;

/**
 * This is an abstract Custom Java Service that serves as a template to all
 * custom services that need to process HTTP requests and respond with HTTP
 * responses. AbstractXQService already provides a good template that abstracts
 * some of the XQ API details, but this service provides even a higher level of
 * abstraction and almost completely releaves developers from heaving to deal
 * with XQMessages. Developers only need to implement familiar doGet and doPost
 * methods. Information in each XQMessage is converted to HttpServletRequest and
 * each response is generated by writing to the HttpServletResponse.
 * 
 * @author Leon Rubin
 */
public abstract class HttpXQServlet extends AbstractXQService {
	private static final String REQUEST_URL_JMS_PROPERTY = "X-HTTP-ReceiveURL";
	private static final String HTTP_REQUEST_METHOD_JMS_PROPERTY = "X-HTTP-RequestMethod";
	private static final String CONTENT_TYPE_JMS_PROPERTY = "Content-Type";
	protected static final String QUERY_STRING_SEPARATOR = "?";
	private static final String HTTP_PARAMETER_SEPARATOR = "&";
	private static final String EQUAL = "=";
	private static final String URL_DELIMITERS = QUERY_STRING_SEPARATOR
			+ HTTP_PARAMETER_SEPARATOR + EQUAL;

	/**
	 * Creates an Http XQ servlet.
	 * 
	 * @param name
	 *            custom service name
	 */
	public HttpXQServlet(String name) {
		super(name);
	}

	/**
	 * This method processes HTTP POST requests because those requests have a
	 * single part that contains the HTTP request body. This method is invoked
	 * by the AbstractXQService for every part that the message has but this
	 * implementation is interested only in part with index 0 and ignores the
	 * rest if they exist. This method extracts JMS properties and the posted
	 * data and represents it as an HttpServletRequest object. Then the
	 * processing is delegated to the doPost method that the subclasses of this
	 * class with implement. It is assumed that the doPost method will write the
	 * response to the HttpServletResponse object and that output must be
	 * returned back to the user. The output generated by the doPost method is
	 * saved back to the original XQPart that used to contain the input.
	 */
	protected void processPart(XQContext ctx, XQEnvelope env, XQMessage msg,
			XQPart part, int partIndex) throws XQServiceException,
			XQMessageException {
		if (partIndex == 0) {
			String httpPostBody = new String((byte[]) part.getContent());
			HttpServletRequestImpl request = createHttpServletRequest(msg,
					httpPostBody);
			request.getRequestURL().append(QUERY_STRING_SEPARATOR);
			request.getRequestURL().append(httpPostBody);

			HttpServletResponseImpl response = new HttpServletResponseImpl();

			doPost(request, response);

			part.setContent(response.getOutput(), response.getContentType());
			msg.replacePart(part, partIndex);
		}
	}

	/**
	 * This method processes HTTP GET requests because those requests have no
	 * pars (no HTTP request body). This method is invoked by the
	 * AbstractXQService once for messages where the part count is 0. This
	 * method extracts only JMS properties and the URL with the query string and
	 * represents it as an HttpServletRequest object. Then the processing is
	 * delegated to the doGet method that the subclasses of this class with
	 * implement. It is assumed that the doGet method will write the response to
	 * the HttpServletResponse object and that output must be returned back to
	 * the user. A new XQPart must be created to contain the output generated by
	 * the doGet method since the original request has no parts.
	 */
	protected void processPartless(XQContext ctx, XQEnvelope env, XQMessage msg)
			throws XQServiceException, XQMessageException {
		HttpServletRequestImpl request = createHttpServletRequest(msg, null);
		HttpServletResponseImpl response = new HttpServletResponseImpl();

		doGet(request, response);

		XQPart part = msg.createPart(response.getOutput(), response
				.getContentType());

		msg.addPart(part);
	}

	/**
	 * Abstract template method that the subclasses are expected to implement to
	 * provide the processing logic for the HTTP GET requests.
	 * 
	 * @param request
	 *            HttpServletRequest created from the user request data
	 * @param response
	 *            HttpServletResponse where the output should be written
	 */
	protected abstract void doGet(HttpServletRequest request,
			HttpServletResponse response);

	/**
	 * Abstract template method that the subclasses are expected to implement to
	 * provide the processing logic for the HTTP POST requests.
	 * 
	 * @param request
	 *            HttpServletRequest created from the user request data
	 * @param response
	 *            HttpServletResponse where the output should be written
	 */
	protected abstract void doPost(HttpServletRequest request,
			HttpServletResponse response);

	/**
	 * Creates the HttpServletRequest from the data contained in the XQMessage
	 * object. This method extracts and populates the original URL request form
	 * the X-HTTP-ReceiveURL JMS property, HTTP method from the
	 * X-HTTP-RequestMethod JMS property, and content type from the Content-Type
	 * JMS property. Even though GET reqeusts store parameters as part of the
	 * URL and POST reqeusts have the same data in the request body, this method
	 * appends the parameter data to the URL before any further processing
	 * regardless the posting method. That way all request appear the same
	 * (parameters are in the query string the URL, so the same parsing logic
	 * can be applied). The logic tokenizes the string using ?, &, and = as
	 * delimiters. All parameters follow the ? character, individual parameters
	 * are separated by & characters, and the = characters separates keys from
	 * values.
	 * 
	 * @param msg
	 *            XQMessage that contains the input data.  
	 * @param httpPostBody
	 *            parameter data posted as part of the POST request or null if
	 *            the data was posted by the GET request
	 * @return HttpServletRequest that encapsulates all parameter data and URL
	 *         information
	 * @throws XQMessageException
	 *             if any errors occur during message processing
	 */
	private HttpServletRequestImpl createHttpServletRequest(XQMessage msg,
			String httpPostBody) throws XQMessageException {
		HttpServletRequestImpl request = new HttpServletRequestImpl();
		request.setRequestURL(msg.getStringHeader(REQUEST_URL_JMS_PROPERTY));

		if (httpPostBody != null && httpPostBody.trim().length() > 0) {
			request.getRequestURL().append(QUERY_STRING_SEPARATOR);
			request.getRequestURL().append(httpPostBody);
		}

		request
				.setMethod(msg
						.getStringHeader(HTTP_REQUEST_METHOD_JMS_PROPERTY));
		request.setContentType(msg.getStringHeader(CONTENT_TYPE_JMS_PROPERTY));

		StringTokenizer st = new StringTokenizer(request.getRequestURL()
				.toString(), URL_DELIMITERS, true);
		Map<String, String> parameterMap = new HashMap<String, String>();
		String previous = null;

		while (st.hasMoreTokens()) {
			String current = st.nextToken();
			if (QUERY_STRING_SEPARATOR.equals(current)
					|| HTTP_PARAMETER_SEPARATOR.equals(current)) {
				// ignore
			} else if (EQUAL.equals(current)) {
				String parameterName = URLDecoder.decode(previous);
				String parameterValue = URLDecoder.decode(st.nextToken());

				System.out.println(parameterName + " == " + parameterValue);

				parameterMap.put(parameterName, parameterValue);
			} else {
				previous = current;
			}
		}

		request.setParameterMap(parameterMap);

		return request;
	}

	/**
	 * This service does not perform any processing on the message level so the
	 * method body is left empty.
	 */
	protected void processMessage(XQServiceContext ctx, XQEnvelope env)
			throws XQServiceException, XQMessageException {

	}
}