package com.onlinebank.transaction.interaction;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.interceptor.ServletRequestAware;

import com.onlinebank.transaction.dao.TransactionDAO;
import com.onlinebank.transaction.dao.TransactionDAOFactory;
import com.onlinebank.transaction.to.TransactionCriteria;
import com.onlinebank.transaction.to.TransactionSearchResult;
import com.onlinebank.util.exception.DAOException;
import com.onlinebank.util.exception.XMLException;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.ModelDriven;

/**
 * This is a Struts2 action that serves as a controller for the transaction
 * search service. The HTTP URL query parameters or the HTTP POST form data are
 * automatically mapped to the TransactionCriteria object that is passed into
 * the DAO (the actual searching is delegated to the Transaction DAO). In
 * Struts2, it is not necessary to create a separate form bean class like in the
 * old version of Struts to encapsulate the form data. Many developers declare
 * the fields for the form data right in the action and the framework saves them
 * right in the action object, however in this, there is an advantate to
 * implementing the ModelDriven interface which mekes Struts2 save the form data
 * to any object returned by the getModel method. Since the real goal of this
 * action is to automatically create the TranactionCriteria from form data, the
 * ModelDriven approach seem more suitable. The getModel will return a blank
 * instance of the TransactionCriteria and the framework will populate it with
 * input data.
 * 
 * @author Leon Rubin
 */
public class FindTransactionsAction extends ActionSupport implements
		ModelDriven<TransactionCriteria>, ServletRequestAware {
	private static final long serialVersionUID = 1L;

	public static final String RESULT_XML_ATTRIBUTE = "resultXml";

	private TransactionCriteria transactionCriteria;
	private HttpServletRequest request;

	/**
	 * Creates the find transaction action.
	 * 
	 */
	public FindTransactionsAction() {
		transactionCriteria = new TransactionCriteria();
	}

	/**
	 * Delegates the search to the findTransactions method of the Transaction
	 * DAO, delegates the conversion of the search result to the Transaction XML
	 * Generat, and saves the XML string to the resultXml request attribute. All
	 * DAO exceptions are converted to action errors.
	 * 
	 * @return SUCCESS if the action executes succesfully or ERROR if an
	 *         exception occurs
	 */
	public String execute() {
		try {
			TransactionDAO dao = TransactionDAOFactory.getDAO();
			TransactionSearchResult result = dao
					.findTransactions(transactionCriteria);
			String resultXml = TransactionXMLGenerator.generate(result);

			request.setAttribute(RESULT_XML_ATTRIBUTE, resultXml);
			return SUCCESS;
		} catch (DAOException e) {
			e.printStackTrace();
			addActionError("Failed to access transaction data " + e);
			return ERROR;
		} catch (XMLException e) {
			e.printStackTrace();
			addActionError("Failed to generate resutl XML " + e);
			return ERROR;
		}
	}

	/**
	 * Sets the request object.
	 * 
	 * @param request
	 *            HTTP servlet request where the XML will be saved
	 */
	public void setServletRequest(HttpServletRequest request) {
		this.request = request;
	}

	/**
	 * Gets the model object of this action which happends to be a transaction
	 * criteria that will be passed to the findTransactions method of the DAO.
	 * 
	 * @return blank TransactionCriteria object that needs to be populated with
	 *         the form data
	 */
	public TransactionCriteria getModel() {
		return transactionCriteria;
	}
}
